[git commit master 1/1] ldso_sh: add support for protected symbols to SH

Carmelo Amoroso carmelo.amoroso at st.com
Fri Sep 24 06:07:41 UTC 2010


commit: http://git.uclibc.org/uClibc/commit/?id=74407db52d3953c7f3c6b8a53661cfc96cb07e22
branch: http://git.uclibc.org/uClibc/commit/?id=refs/heads/master

Protected symbols should not be overridden by
symbols from other modules. Such symbols are exported
i.e. globally visible, but references from whithin
defining modules are satisfied locally.

Signed-off-by: Salvatore Cro <salvatore.cro at st.com>
Signed-off-by: Carmelo Amoroso <carmelo.amoroso at st.com>
---
 ldso/ldso/sh/elfinterp.c |   42 ++++++++++++++++++++++++------------------
 1 files changed, 24 insertions(+), 18 deletions(-)

diff --git a/ldso/ldso/sh/elfinterp.c b/ldso/ldso/sh/elfinterp.c
index 715eadc..756f6c4 100644
--- a/ldso/ldso/sh/elfinterp.c
+++ b/ldso/ldso/sh/elfinterp.c
@@ -166,26 +166,32 @@ _dl_do_reloc (struct elf_resolve *tpnt,struct dyn_elf *scope,
 	reloc_type = ELF32_R_TYPE(rpnt->r_info);
 	symtab_index = ELF32_R_SYM(rpnt->r_info);
 	symbol_addr = 0;
-	symname = strtab + symtab[symtab_index].st_name;
 
 	if (symtab_index) {
-		symbol_addr = (unsigned long) _dl_find_hash(symname, scope, tpnt,
-							    elf_machine_type_class(reloc_type), &tls_tpnt);
-		/*
-		 * We want to allow undefined references to weak symbols - this might
-		 * have been intentional.  We should not be linking local symbols
-		 * here, so all bases should be covered.
-		 */
-
-		if (!symbol_addr
-			&& (ELF_ST_TYPE(symtab[symtab_index].st_info) != STT_TLS)
-			&& (ELF32_ST_BIND(symtab[symtab_index].st_info) != STB_WEAK)) {
-			_dl_dprintf(2, "%s: can't resolve symbol '%s'\n",
-			            _dl_progname, strtab + symtab[symtab_index].st_name);
-
-			/* Let the caller to handle the error: it may be non fatal if called from dlopen */
-			return 1;
-		}
+		symname = strtab + symtab[symtab_index].st_name;
+		if (ELF32_ST_VISIBILITY(symtab[symtab_index].st_other)
+			 != STV_PROTECTED) {
+			symbol_addr = (unsigned long) _dl_find_hash(symname, scope, tpnt,
+								elf_machine_type_class(reloc_type), &tls_tpnt);
+			/*
+			 * We want to allow undefined references to weak symbols - this might
+			 * have been intentional.  We should not be linking local symbols
+			 * here, so all bases should be covered.
+			 */
+
+			if (!symbol_addr
+				&& (ELF_ST_TYPE(symtab[symtab_index].st_info) != STT_TLS)
+				&& (ELF32_ST_BIND(symtab[symtab_index].st_info) != STB_WEAK)) {
+				_dl_dprintf(2, "%s: can't resolve symbol '%s'\n",
+				            _dl_progname, strtab + symtab[symtab_index].st_name);
+
+				/* Let the caller to handle the error: it may be non fatal if called from dlopen */
+				return 1;
+			}
+		} else
+			/* Resolve protected symbols locally */
+			symbol_addr = DL_FIND_HASH_VALUE(tpnt, elf_machine_type_class(reloc_type),
+											&symtab[symtab_index]);
 	}
 
 #if defined (__SUPPORT_LD_DEBUG__)
-- 
1.7.1



More information about the uClibc-cvs mailing list